%
%
%  3D to 2D Projection Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Project to 2D" icon="Matlab" tooltip="Resize the dataset to 2 dimensions.">
%          <Command>MatlabXT::XT3Dto2DProjection(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%
%
%  Description:
%
%   Resize the dataset to 2 dimensions.
%   2 dialog boxes ask the projection plane XY, XZ or YZ) and the 
%       projection mode (MIP or Mean) if their values are not specified 
%       as input parameters.
%
%

function XT3Dto2DProjection(aImarisApplicationID, aProjectionPlane, aProjectionMode)


if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end
vImarisApplication.DataSetPushUndo('Projection to 2D');

% get the data set 
vDataSet = vImarisApplication.GetDataSet.Clone;

vNumberOfTimePoints = vDataSet.GetSizeT;
vNumberOfChannels = vDataSet.GetSizeC;

% projection plane
vPlaneNames = ['XY'; 'XZ'; 'YZ'];

if nargin<2
    [vAnswer, vOk] = listdlg('ListString',{vPlaneNames},...
                           'SelectionMode','single',...
                           'ListSize',[200 50],...
                           'Name','3D to 2D Projection',...
                           'PromptString',{'Please select  the projection plane:'});
    if vOk<1
        return;
    end

    vProjectionPlane = vAnswer;
else
    vProjectionPlane = aProjectionPlane;
end
vProjectionAxis = 4-vProjectionPlane;

% MIP or Mean projection
if nargin<3
    vAnswer = questdlg('Please select the projection mode:', '3D to 2D Projection', ...
        'MIP', 'Mean','Cancel', 'MIP');
    if strcmp(vAnswer, 'Cancel')
        return;
    else
        vProjectionMode = vAnswer;
    end
else
    vProjectionMode = aProjectionMode;
end

vTotalCount = vNumberOfChannels*vNumberOfTimePoints;
vProgressDisplay = waitbar(0,['Projecting to ',vPlaneNames(vProjectionPlane,:),'-plane']);
vProgressCount = 0;

vSize = [vDataSet.GetSizeX,vDataSet.GetSizeY,vDataSet.GetSizeZ];

for vChannel = 1:vNumberOfChannels
    for vTime = 1:vNumberOfTimePoints
        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vImage = typecast(vDataSet.GetDataVolumeAs1DArrayBytes( ...
                vChannel-1,vTime-1), 'uint8');
        elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
            vImage = typecast(vDataSet.GetDataVolumeAs1DArrayShorts( ...
                vChannel-1,vTime-1), 'uint16');
        elseif strcmp(vDataSet.GetType, 'eTypeFloat')
            vImage = vDataSet.GetDataVolumeAs1DArrayFloats( ...
                vChannel-1,vTime-1);
        end
        vImage = reshape(vImage, vSize);
        
        if strcmp(vProjectionMode, 'MIP')
            % MIP
            vImageSlice = max(vImage, [], vProjectionAxis); 
        else
            % Mean 
            vImageSlice = mean(vImage, vProjectionAxis);     
        end

        % original size must be preserved because resizing the first layer
        % would resize every layer (losing data)
        switch vProjectionPlane
            case 1,
                vImage(:,:,1) = vImageSlice;
            case 2,
                vImage(:,1,:) = vImageSlice;
            case 3,
                vImage(1,:,:) = vImageSlice;
        end

        vProgressCount = vProgressCount + 1;
        waitbar(vProgressCount/vTotalCount)

        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vDataSet.SetDataVolumeBytes(vImage,vChannel-1,vTime-1);
        elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
            vDataSet.SetDataVolumeShorts(vImage,vChannel-1,vTime-1);
        elseif strcmp(vDataSet.GetType, 'eTypeFloat')
            vDataSet.SetDataVolumeFloats(vImage,vChannel-1,vTime-1);
        end
    end
end

% resize layers
switch vProjectionPlane
    case 1,
        vDataSet.SetSizeZ(1);
    case 2,
        vDataSet.SetSizeY(1);
    case 3,
        vDataSet.SetSizeX(1);
end 

vImarisApplication.SetDataSet(vDataSet);

close(vProgressDisplay);

